//+------------------------------------------------------------------+
//|                                  Kaufman Adaptive Moving Average |
//|                                            Kaufman AMA basic.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    AMAPeriod       = 10;
extern int    AMAPrice        = PRICE_CLOSE;
extern int    Nfast           = 2;
extern int    Nslow           = 30;
extern double GCoeff          = 2;
extern int    PriceFilter     = 5;
extern int    PriceFilterMode = MODE_EMA;

//
//
//
//
//

double kAMAbuffer[];
double kAMAbufferda[];
double kAMAbufferdb[];
double wAMAbuffer[];
double diff[];
double slope[];

//
//
//
//
//

double fastend;
double slowend;
string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,kAMAbuffer);
   SetIndexBuffer(1,kAMAbufferda);
   SetIndexBuffer(2,kAMAbufferdb);
   SetIndexBuffer(3,wAMAbuffer);
   SetIndexBuffer(4,diff);
   SetIndexBuffer(5,slope);
   
   //
   //
   //
   //
   //
   
      fastend           = (2.0 /(Nfast + 1));
      slowend           = (2.0 /(Nslow + 1));
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+"  Kaufman AMA ("+AMAPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars)  { kAMAbuffer[0] = limit+1; return(0); } 

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {  
     if (slope[limit]==-1) CleanPoint(limit,kAMAbufferda,kAMAbufferdb);
     for(int i=limit; i>=0; i--)
     {
        kAMAbuffer[i]   = iKama(kAMAbuffer,diff,AMAPeriod,AMAPrice,GCoeff,i);
        kAMAbufferda[i] = EMPTY_VALUE;
        kAMAbufferdb[i] = EMPTY_VALUE;
        slope[i]        = slope[i+1];
           if (kAMAbuffer[i]>kAMAbuffer[i+1]) slope[i] =  1;
           if (kAMAbuffer[i]<kAMAbuffer[i+1]) slope[i] = -1;
           if (slope[i]==-1) PlotPoint(i,kAMAbufferda,kAMAbufferdb,kAMAbuffer);
     }      
     return(0);
   }
   
   //
   //
   //
   //
   //
   
    limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
    if (slope[limit]==-1) CleanPoint(limit,kAMAbufferda,kAMAbufferdb);
    for(i=limit; i >= 0; i--)  
    {
       int y = iBarShift(NULL,timeFrame,Time[i]);
          kAMAbuffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AMAPeriod,AMAPrice,Nfast,Nslow,GCoeff,PriceFilter,PriceFilterMode,0,y);
          slope[i]        = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",AMAPeriod,AMAPrice,Nfast,Nslow,GCoeff,PriceFilter,PriceFilterMode,5,y);
          kAMAbufferda[i] = EMPTY_VALUE;
          kAMAbufferdb[i] = EMPTY_VALUE;
   }
   for (i=limit;i>=0;i--)  if (slope[i]==-1) PlotPoint(i,kAMAbufferda,kAMAbufferdb,kAMAbuffer);  
   return(0);
}
  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

double price[];
double iKama(double& ama_buffer[],double& diff_buffer[],int period,int priceType,double gCoeff,int i)
{
   double efratio  = 1.00;
   double AMA      = 0.00;
   if (ArraySize(price)!=Bars) ArrayResize(price,Bars); int r = Bars-i-1;
      price[r]     = iMA(NULL,0,PriceFilter,0,PriceFilterMode,priceType,i);


   if (i > Bars-period) return(price[r]);
   
   //
   //
   //
   //
   //
   
   double smooth;
   double signal;
   double noise;
          signal         = MathAbs(price[r]-price[r-period]);
          diff_buffer[i] = MathAbs(price[r]-price[r-1]);
          for (int k=0;k<period;k++)
                  noise += diff_buffer[i+k];

          //
          //
          //
          //
          //

          if (noise != 0) efratio = signal/noise;
                          smooth  = MathPow(efratio*(fastend-slowend)+slowend,gCoeff);
                          AMA     = ama_buffer[i+1] + smooth*(price[r]-ama_buffer[i+1]);
   //
   //
   //
   //
   //

   return(AMA);          
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
